
from enpassant import *

class WrongAnswer(ValueError):
    """
    Raised when test takes wrong fork in a conditional.
    """
    pass

def test_one():
    passer = Passer()
    values = [ 1, 0, True, False, "yes", "", [], object() ]

    for v in values:
        if passer / v:
            assert bool(passer.value) is True
            assert passer.value == v
        else:
            assert bool(passer.value) is False
            assert passer.value == v

def test_bool():
    passer = Passer()
    values = [ 1, 0, True, False, "yes", "", [], object() ]
    bools  = [ True, False, True, False, True, False, False, True]

    for v, b in zip(values, bools):
        passer / v
        assert bool(passer) == b


def test_unary_value_retrieval():
    passer = Passer()
    values = [ 1, 0, True, False, "yes", "", [], object() ]

    for v in values:
        if passer / v:
            assert bool(+passer) is True
            assert +passer == v
        else:
            assert bool(passer.value) is False
            assert bool(+passer) is False
            assert +passer == v

    class AttrDict(dict):
        """
        Simple attributes-exposed dict class for testing purposes.
        Use stuf, TreeDict, or something better in real apps.
        """
        def __getattr__(self, name):
            return self.__getitem__(name)

    d = AttrDict(a=1, b=2, c=lambda x: x * 2)
    if passer / d:
        assert +d.a == 1
        assert +d.b == 2
        assert +d.c(10) == 20
        assert +d['a'] == 1
        assert +d['b'] == 2
        assert +d['c'](10) == 20
    else:
        assert False


def test_alt_operations():
    passer = Passer()

    values = [ 1, 0, True, False, "yes", "", [], object() ]

    for v in values:
        if passer < v:
            assert bool(passer.value) is True
            assert passer.value == v
        else:
            assert bool(passer.value) is False
            assert passer.value == v

    for v in values:
        if passer <= v:
            assert bool(passer.value) is True
            assert passer.value == v
        else:
            assert bool(passer.value) is False
            assert passer.value == v

def test_call_delivery():
    passer = Passer()

    values = [ 1, 0, True, False, "yes", "", [], object() ]

    for v in values:
        if passer <= v:
            assert bool(passer.value) is True
            assert passer() == v
        else:
            assert bool(passer.value) is False
            assert passer() == v


def test_call_capture():
    passer = Passer()

    values = [ 1, 0, True, False, "yes", "", [], object() ]

    for v in values:
        if passer(v):
            assert bool(passer.value) is True
            assert passer.value == v
            assert passer() == v
        else:
            assert bool(passer.value) is False
            assert passer.value == v
            assert passer() == v


def test_item_forwarding():
    answers = [ 1, 4, 'a']
    result = Passer()

    if result / answers:
        assert result[0] == 1
        assert result[1] == 4
        assert result[2] == 'a'
        assert result.value is answers
    else:
        raise WrongAnswer()

def test_attribute_forwarding():
    class X(object):
        one = 1
        two = 2
        more = 'more'

        def report(self):
            return "{0} is {1} than {2}".format(self.two, self.more, self.one)

    result = Passer()
    x = X()
    if result/x:
        assert result.one == 1
        assert result.two == 2
        assert result.more == 'more'
        assert result.value is x
        assert result.report() == '2 is more than 1'
    else:
        raise WrongAnswer()

def test_item_setting():
    answers = [ 1, 4, 'a']
    result = Passer()

    if result / answers:
        result[2] = 'two'
        result.append('extra')
        assert result[0] == 1
        assert result[1] == 4
        assert result[2] == 'two'
        assert result[3] == 'extra'
        assert len(result.value) == 4
        assert result.value is answers
    else:
        raise WrongAnswer()

def test_attribute_setting():
    class X(object):
        one = 1
        two = 2
        more = 'more'

        def report(self):
            return "{0} is {1} than {2}".format(self.two, self.more, self.one)

    result = Passer()
    x = X()
    if result/x:
        result.one = 'one'
        result.two = 'two'
        assert result.one == 'one'
        assert result.two == 'two'
        assert result.more == 'more'
        assert result.value is x
        assert result.report() == 'two is more than one'
    else:
        raise WrongAnswer()

def test_repr():
    values = [ 1, 0, True, False, "yes", "", [] ]
    passer = Passer()

    for v in values:
        if passer / v:
            pass
        assert repr(passer) == 'Passer(' + repr(v) + ')'


def test_grabber():
    info = Grabber()

    if info.name("roger"):
        assert info.name == 'roger'
    if info.empty([]):
        assert False
    else:
        assert info.empty == []


def test_grabber_repr():
    info = Grabber()
    info.name("roger")
    info.age(12)
    info.color("orange")
    assert repr(info) == "Grabber(name='roger', age=12, color='orange')"




# Test ordereddict sufficiently for its role as a Python 2.6 standin

import pytest
import sys
_PY26 = sys.version_info[:2] == (2, 6)
if _PY26:
    from enpassant.ordereddict import *


pytestmark = pytest.mark.skipif(not _PY26, reason='unnecessary')


def test_bad_constructor():
    with pytest.raises(TypeError):
        OrderedDict(1, 3, 4)


def test_delitem():
    od = OrderedDict()
    od['a'] = 4
    od['b'] = 55
    od['c'] = 323
    assert list(od.keys()) == list('abc')
    del od['b']
    assert list(od.keys()) == list('ac')


def test_reversed():
    od = OrderedDict()
    od['a'] = 4
    od['b'] = 55
    od['c'] = 323

    assert list(reversed(od)) == list('cba')


def test_eq():
    od = OrderedDict()
    od['a'] = 4
    od['b'] = 55
    od['c'] = 323

    od2 = OrderedDict()
    od2['a'] = 4
    od2['b'] = 55
    od2['c'] = 323

    assert od == od2

    od2['d'] = 42
    assert od != od2

    od3 = OrderedDict()
    od3['b'] = 55
    od3['a'] = 4
    od3['c'] = 323

    assert od != od3

    d3 = dict(od3.items())
    assert od == d3


def test_copy():
    od = OrderedDict()
    od['a'] = 4
    od['b'] = 55
    od['c'] = 323

    od2 = od.copy()

    assert od == od2
    assert od is not od2


def test_repr():
    od = OrderedDict()
    od['a'] = 4
    od['b'] = 55
    od['c'] = 323

    assert repr(od) == "OrderedDict([('a', 4), ('b', 55), ('c', 323)])"
    assert repr(OrderedDict()) == "OrderedDict()"


def test_fromkeys():
    od = OrderedDict.fromkeys(list('abc'), 33)
    assert list(od.keys()) == list('abc')
    assert list(od.values()) == [33] * 3


def test_popitem():
    od = OrderedDict()
    od['a'] = 4
    od['b'] = 55
    od['c'] = 323

    with pytest.raises(KeyError):
        OrderedDict().popitem()

    assert od.popitem(last=True) == ('c', 323)
    assert od.popitem(last=False) == ('a', 4)

def test_reduce():
    import pickle
    od = OrderedDict()
    od['a'] = 4
    od['b'] = 55
    od['c'] = 323

    od1 = pickle.loads(pickle.dumps(od))

    assert od == od1



"""
Define en passant helper objects.
"""

import sys
_PY3 = sys.version_info[0] > 2

try:
    from collections import OrderedDict
except ImportError:
    # Should only be an issue for Python <= 2.6
    from .ordereddict import OrderedDict

__all__ = 'Passer Grabber'.split()


class _NeverSeen(object):
    pass


class Passer(object):

    """
    Objects that help proxy values for en passant operations.
    """

    def __init__(self):
        """
        Make one!
        """
        self.value = _NeverSeen

    def _grab(self, other):
        """
        Get the value of the second operand, and return it as well.
        """
        self.value = other
        return other

    def _bool(self):
        """
        Return the boolean value of our value.
        """
        return bool(self.value)

    if _PY3:
        __truediv__ = _grab
        __bool__ = _bool
    else:
        __div__ = _grab
        __nonzero__ = _bool

    # define alterate operators (< and <=)
    __lt__ = _grab
    __le__ = _grab

    def __getattr__(self, name):
        """
        Forward attribute access to the contained value.
        """
        try:
            return object.__getattr__(self, name)
        except AttributeError:
            return getattr(self.value, name)

    def __setattr__(self, name, value):
        """
        Forward attribute setting to the contained value.
        """
        if name == 'value':
            object.__setattr__(self, name, value)
        else:
            setattr(self.value, name, value)

    def __getitem__(self, n):
        """
        Forward item access (indexing) to the contained value.
        """
        return self.value.__getitem__(n)

    def __setitem__(self, n, value):
        """
        Forward item setting (via index) to the contained value.
        """
        return self.value.__setitem__(n, value)

    def __call__(self, arg=_NeverSeen):
        """
        Use call to either caputre a result value or render the
        contained value. The first time called, captures. On
        subsequent calls, renders.
        """
        if arg is not _NeverSeen:
            self.value = arg
        return self.value

    def __pos__(self):
        """
        Implements positive unary operation, +x, to render value.
        """
        return self.value

    def __repr__(self):
        """
        Return string representation, showing current value.
        """
        return "{0}({1!r})".format(self.__class__.__name__, self.value)


class Grabber(object):

    """
    Objects that grab the value of initial attribute calls.
    """

    def __init__(self):
        """
        Make one!"
        """
        self._data = OrderedDict()

    def __getattr__(self, name):
        """
        The attribute call. If no such attribute already exists, return a
        setter function that will do the inital setting.
        """
        if name in self._data:
            return self._data[name]
        else:
            def setter(value):
                self._data[name] = value
                return value
            return setter

    def __repr__(self):
        """
        Return the string representation.
        """
        guts = ', '.join("{0}={1!r}".format(k, v)
                         for k, v in self._data.items())
        return "{0}({1})".format(self.__class__.__name__, guts)
